import json
from datetime import datetime

dta = {}

def output(pth):
    with open(pth, 'r') as f:
        dta = json.load(f)
    n = len(dta["lines"])
    print("----------------------------------------------")
    print("                     REPORT                   ")
    print("----------------------------------------------")
    print("                     SOURCE                   ")
    print("----------------------------------------------")
    print(dta["source"])
    print("----------------------------------------------")
    print("                     LINES                    ")
    print("----------------------------------------------")
    for i in range(1, n+1):
        print("At time %s," % (datetime.fromtimestamp(dta["lines"][str(i)]["timestamp"])), end = " ")
        curr = dta["lines"][str(i)]["report"]
        print("starting line %s" % (curr[0]))
        print("The time spent on this line is {:f} seconds".format(curr[1]))
        print("This line has been executed %s times before." % (curr[2]))
        print("The aggregate time spent on this line till now is {:f} seconds".format(curr[3]))
        print("The average time spent on this line till now is {:f} seconds".format(curr[4]))
        for j in curr[5]:
            varchanges = "In line %s, the value %s has been added to %s" % (curr[0], j[1], j[0])
            print(varchanges)
        for j in curr[6]:
            varchanges = "In line %s, the index %s of list %s has changed value from %s to %s" % (curr[0], j[1], j[0], j[2], j[3])
            print(varchanges)
        for j in curr[7]:
            varchanges = "In line %s, the variable %s has been initialized to %s" % (curr[0], j[0], j[1])
            print(varchanges)
        for j in curr[8]:
            varchanges = "In line %s, the variable %s has been changed value from %s to %s" % (curr[0], j[0], j[1], j[2])
            print(varchanges)
        for j in curr[9]:
            varchanges = "In line %s, the variable %s has been initialized to %s" % (curr[0], j[0], j[1])
            print(varchanges)
        print("----------------------------------------------")
    print("                   VARIABLES                  ")
    print("----------------------------------------------")
    for i in dta["variables"]:
        print("Tracing the variable " + i + " of type %s:-" % (dta["variables"][i]["type"])) 
        print("It was initialized on line %s" % (dta["variables"][i]["report"][1]))
        if(dta["variables"][i]["type"] == "int"):
            mnval = dta["variables"][i]["report"][2]
            mxval = dta["variables"][i]["report"][2]
            lastval = dta["variables"][i]["report"][2]
            print("The initial value of the variable was %s" % (lastval))
            for j in dta["variables"][i]["report"][3]:
                print("In line %s, the variable %s changed values from %s to %s"%(j[0], i, j[1], j[2]))
                lastval = j[2]
                mnval = min(j[2], mnval)
                mxval = max(j[2], mxval)
            print("The value of the variable ranges from %s to %s" % (mnval, mxval))
            print("The final value of the variable is %s" % (lastval))
        else: 
            lastval = dta["variables"][i]["report"][2]
            print("The initial value of the variable was %s" % (lastval))
            for j in dta["variables"][i]["report"][3]:
                print("In line %s, the variable %s changed values from %s to %s"%(j[0], i, j[1], j[2]))
                lastval = j[2]
            print("The final value of the variable is %s" % (lastval))
        print("----------------------------------------------")
    cnt= 0
    print("               OTHER INFORMATION              ")
    print("----------------------------------------------")
    for i in dta["others"]["report"]:
        cnt+=1
        if(cnt == 1):
            print("The program finished in {:f} seconds".format(curr[3]))
            print("----------------------------------------------")
        else:
            if int(i[1]) != 1:
                print("Line %s was executed %s times" % (i[0], i[1]))
            else:
                print("Line %s was executed %s time" % (i[0], i[1]))
    print("----------------------------------------------")


adj= {}
visited = []

def initiliazegraph():
    n = 8
    for i in range(0, n+1):
        visited.append(False)
        adj[i] = []
    adj[1].append(2)
    adj[1].append(4)
    adj[4].append(3)
    adj[4].append(5)
    adj[2].append(1)
    adj[4].append(1)
    adj[3].append(4)
    adj[5].append(4)
    adj[6].append(7)
    adj[7].append(8)
    adj[7].append(6)
    adj[8].append(7)
    adj[2].append(6)
    adj[6].append(2)
    

def quicksort(arr):
    l = []
    equal = []
    r = []
    if len(arr) > 1:
        pvt = arr[0]
        for x in arr:
            if x < pvt:
                l.append(x)
            elif x == pvt:
                equal.append(x)
            elif x > pvt:
                r.append(x)
        arr = quicksort(l)+equal+quicksort(r)
        return arr
    else:
        return arr

def binarysearch(arr, l, r, x):
    if r >= l: 
        mid = int(l + (r - l)/2)
        if arr[mid] == x: 
            return mid 
        elif arr[mid] > x: 
            return binarysearch(arr, l, mid-1, x) 
        else: 
            return binarysearch(arr, mid+1, r, x) 
    else: 
        return -1

def dfs(adj, visited, s):
    visited[s] = True
    for u in adj[s]:
        if not visited[u]:
            dfs(adj, visited, u)

def bfs(adj, visited, s):
    q = []
    q.append(s)
    while len(q) != 0:
        u = q[0]
        visited[u]= True
        q.pop(0)
        for v in adj[u]:
            if not visited[v]:
                q.append(v)

def knapsack(left, wt, val, n):
    if n == 0 or left == 0 : 
        return 0
    if (wt[n-1] > left): 
        return knapsack(left, wt, val, n-1) 
    else: 
        return max(val[n-1] + knapsack(left-wt[n-1], wt, val, n-1), knapsack(left, wt, val, n-1)) 

def bubblesort(arr):
    n = len(arr)
    for i in range(n):
        for j in range(0, n-i-1):
            if arr[j] > arr[j+1]:
                arr[j], arr[j+1] = arr[j+1], arr[j]
    return arr

def lis(arr):
    dp = []
    for i in range(len(arr)):
        dp.append(i)
    for i in range(len(arr)):
        for j in range(0, i):
            if(arr[j] < arr[i] and dp[i] < dp[j]+1):
                dp[i] = dp[j]+1
    ans =1 
    for i in dp:
        ans= max(i, ans)
    return ans

def lcs(s, t, n, m):
    if m == 0 or n == 0: 
       return 0
    elif t[m-1] == s[n-1]: 
       return 1 + lcs(s, t, n-1, m-1)
    else: 
       return max(lcs(s, t, n, m-1), lcs(s, t, n-1, m))

def insertionsort(arr):
    for i in range(1, len(arr)): 
        key = arr[i] 
        j = i-1
        while j >= 0 and key < arr[j]: 
                arr[j+1] = arr[j] 
                j -= 1
        arr[j+1] = key 
    return arr

def kadanes(arr):
    curr = 0
    maxans = 0
    for i in range(len(arr)):
        curr += arr[i]
        curr = max(curr, 0)
        maxans = max(curr, maxans)
    return maxans


import inspect
import sys
import copy
import time
import timeit
import importlib
import importlib.util
from . import writetojson, outputjson, generatevid, testsuite
# import writetojson
# import outputjson
from inspect import isfunction, getmembers
# import generatevid
# import testsuite

variablevalues = {}
answerint = {}
answerother = {}
aggtimeperline = {}
nooftimesperline = {}

og = sys.stdout
step = 0

def trace_main(frame, event, args):
    if(event != 'call'):
        return 
    return trace_varchanges
lastoutput = []
def trace_varchanges(frame, event, args):
    global step
    global lastoutput
    global og
    if event != "line":
        return
    code = frame.f_code
    localvars = frame.f_locals
    line_no = frame.f_lineno
    arr = []
    strt = "Starting line %s ..." % (line_no)
    arr.append(line_no)
    start_time = time.time()
    step+=1
    vrchngs = []
    vrinitialize = []
    lstchngs = []
    lstadds= []
    lstinitialize = []
    sys.stdout = og
    for i in code.co_varnames:
        if i in localvars:
            if(isinstance(localvars[i], list)):
                if i not in variablevalues:
                    line_no = frame.f_lineno
                    func_name = code.co_name
                    lstinitialize.append([i, localvars[i]])
                    variablevalues[i] = []
                    variablevalues[i] = localvars[i][:]
                    answerother[i] = [line_no, type(localvars[i]), [[line_no, variablevalues[i]]]]
                else:
                    changed = False
                    if (len(variablevalues[i]) < len(localvars[i])):
                        line_no = frame.f_lineno
                        func_name = code.co_name
                        for j in range(len(variablevalues[i])):
                            if(variablevalues[i][j] != localvars[i][j]):        
                                lstchngs.append([i, j, variablevalues[i][j], localvars[i][j]])
                                changed = True
                        for j in range(len(variablevalues[i]), len(localvars[i])):
                            lstadds.append([i, localvars[i][j]])
                            changed = True
                        variablevalues[i] = []
                        variablevalues[i] = localvars[i][:]
                    else:
                        line_no = frame.f_lineno
                        func_name = code.co_name
                        for j in range(min(len(variablevalues[i]), len(localvars[i]))):
                            if(variablevalues[i][j] != localvars[i][j]):        
                                lstchngs.append([i, j, variablevalues[i][j], localvars[i][j]])
                                changed = True
                        variablevalues[i] = []
                        variablevalues[i] = localvars[i][:]
                    if changed == True:
                        answerother[i][2].append([line_no, variablevalues[i]])
            else:
                if i not in variablevalues:
                    line_no = frame.f_lineno
                    func_name = code.co_name
                    vrinitialize.append([i, localvars[i]])
                    if(isinstance(localvars[i], int)):
                        answerint[i] = [line_no, localvars[i], localvars[i], [[line_no, localvars[i]]]]
                    else:
                        if(isinstance(localvars[i], tuple) or isinstance(localvars[i], dict) or isinstance(localvars[i], set)):
                            temp = localvars[i].copy()
                        else:
                            temp = localvars[i]
                        answerother[i] = [line_no, type(localvars[i]), [[line_no, temp]]]
                    if(isinstance(localvars[i], tuple) or isinstance(localvars[i], dict) or isinstance(localvars[i], set)):
                        variablevalues[i] = localvars[i].copy()
                    else:
                        variablevalues[i] = localvars[i]
                else:
                    if(localvars[i] != variablevalues[i]):
                        line_no = frame.f_lineno
                        func_name = code.co_name
                        vrchngs.append([i, variablevalues[i], localvars[i]])
                        if(isinstance(localvars[i], int)):
                            answerint[i][1] = min(answerint[i][1], localvars[i])
                            answerint[i][2] = max(answerint[i][2], localvars[i])
                            answerint[i][3].append([line_no, localvars[i]])
                        else:
                            if(isinstance(localvars[i], tuple) or isinstance(localvars[i], dict) or isinstance(localvars[i], set)):
                                temp = localvars[i].copy()
                            else:
                                temp = localvars[i]
                            answerother[i][2].append([line_no, temp])
                            
                        if(isinstance(localvars[i], tuple) or isinstance(localvars[i], dict) or isinstance(localvars[i], set)):
                            variablevalues[i] = localvars[i].copy()
                        else:
                            variablevalues[i] = localvars[i]
    line_no = frame.f_lineno
    if line_no not in nooftimesperline:
        nooftimesperline[line_no] = 1
    else:
        nooftimesperline[line_no]+=1
    spent = time.time() - start_time
    if line_no not in aggtimeperline:
        aggtimeperline[line_no] = spent
    else:
        aggtimeperline[line_no]+=spent
    arr.append(spent)
    arr.append(nooftimesperline[line_no])
    arr.append(aggtimeperline[line_no])
    arr.append(aggtimeperline[line_no]/nooftimesperline[line_no])
    arr.append(lstadds)
    arr.append(lstchngs)
    arr.append(lstinitialize)
    arr.append(vrchngs)
    arr.append(vrinitialize)
    file = open("output.txt", "r")
    otpt = ""
    curroutput = list(file.read())
    if(len(lastoutput) < len(curroutput)):
        for i in range(len(lastoutput), len(curroutput)):
            otpt += curroutput[i]
    arr.append(otpt)
    lastoutput = curroutput.copy()
    file.close()
    jsndta = {}
    jsndta["timestamp"] = time.time()
    jsndta["report"] = arr
    writetojson.addtolines(jsndta, step)
    sys.stdout = open("output.txt", "a")
def main():
    print("----------------------------------------------")
    print("                 TAKING INPUT                 ")
    print("----------------------------------------------")
    tests = False
    functest = 0
    a1 = input("Do you want to use the test suite? (y/n): ")
    if a1 == 'y':
        tests = True
        print("Which algorithm do you want to test?")
        print("1 - Quicksort")
        print("2 - Binary Search")
        print("3 - Depth-First Search")
        print("4 - Breadth-First Search")
        print("5 - Knapsack Problem")
        print("6 - Bubble Sort")
        print("7 - Longest Increasing Subsequence")
        print("8 - Longest Common Subsequence")
        print("9 - Insertion Sort")
        print("10 - Kadanes Algorithm")
        t1= int(input("Enter: "))
        while(t1 > 10 or t1<1):
            t1 = int(input("Sorry invalid input! Enter Again:"))
        functest = t1
    else:
        pth2 = input("Enter the name of the file you want to debug: ")
        if pth2.find(".json") != -1:
            inp2 = input("Do you want to generate a video? (y/n): ")
            if(inp2 == 'y'):
                print("Generating video...")
                generatevid.generatevid(pth2)
                inp3 = input("Do you want to generate a GIF file as well? (y/n)")
                if(inp3 == "y"):
                    print("Generating GIF...")
                    generatevid.convertogif()
                exit()
            print("You entered a JSON file, reporting the data...")
            outputjson.output(pth2)
            exit()
        if pth2.find(".py") == -1:
            print("Invalid file, breaking the program")
            exit()
        fname = input("Enter the name of the function you want to debug: ")
        pth = pth2.replace(".py", "")
        spec = importlib.util.spec_from_file_location("func", pth2)
        foo = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(foo)
        if(not hasattr(foo, fname)):
            print("Invalid input! Either wrong function name or wrong path!")
            exit()
    args = list(input("Enter the arguments for the function (space seperated): ").split())
    for i in range(len(args)):
        args[i] = int(args[i])
    print("----------------------------------------------")
    print("              RUNNING FUNCTION                ")
    print("----------------------------------------------")


    stime = time.time()
    if(tests == False):
        spec = importlib.util.spec_from_file_location("func", pth2)
        foo = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(foo)
        func = getattr(foo, fname)
        writetojson.addtosource(inspect.getsource(func))
    else:
        testsuite.initiliazegraph()
        if functest== 1:
            testsuite.quicksort([4, 8, 5, 6, 2, 10])
            writetojson.addtosource(inspect.getsource(testsuite.quicksort))
        elif functest==2:
            writetojson.addtosource(inspect.getsource(testsuite.binarysearch))
        elif functest==3:
            writetojson.addtosource(inspect.getsource(testsuite.dfs))
        elif functest==4:
            writetojson.addtosource(inspect.getsource(testsuite.bfs))
        elif functest==5:
            writetojson.addtosource(inspect.getsource(testsuite.knapsack))
        elif functest==6:
            writetojson.addtosource(inspect.getsource(testsuite.bubblesort))
        elif functest==7:
            writetojson.addtosource(inspect.getsource(testsuite.lis))
        elif functest==8:
            writetojson.addtosource(inspect.getsource(testsuite.lcs))
        elif functest==9:
            writetojson.addtosource(inspect.getsource(testsuite.insertionsort))
        elif functest==10:
            writetojson.addtosource(inspect.getsource(testsuite.kadanes))
    sys.stdout = open("output.txt", "w")
    sys.settrace(trace_main)
    if(tests == True):
        if functest== 1:
            testsuite.quicksort([4, 8, 5, 6, 2, 10])
        elif functest==2:
            testsuite.binarysearch([2, 6, 9, 10, 14], 0, 4, 6)
        elif functest==3:
            testsuite.dfs(testsuite.adj, testsuite.visited, 1)
        elif functest==4:
            testsuite.bfs(testsuite.adj, testsuite.visited, 1)
        elif functest==5:
            testsuite.knapsack(50, [10,20,30], [60,100,120], 3)
        elif functest==6:
            testsuite.bubblesort([4, 8, 5, 6, 2, 10])
        elif functest==7:
            testsuite.lis([4, 8, 5, 6, 2, 10])
        elif functest==8:
            testsuite.lcs("AGGTAB", "GXTXAYB", 6, 7)
        elif functest==9:
            testsuite.insertionsort([4, 8, 5, 6, 2, 10])
        elif functest==10:
            testsuite.kadanes([-2, -3, 4, -1, -2, 1, 5, -3])
    else:
        func(*args)

    sys.settrace(None)
    sys.stdout = og
    arr = []
    for i in answerint:
        arr = []
        arr.append(i)
        arr.append(answerint[i][0])
        lastval = answerint[i][3][0][1]
        arr.append(lastval)
        arr2 = []
        for j in range(1, len(answerint[i][3])):
            arr2.append([answerint[i][3][j][0], lastval, answerint[i][3][j][1]])
            lastval = answerint[i][3][j][1]
        arr.append(arr2)
        jsndta = {}
        jsndta["timestamp"] = time.time()
        jsndta["report"] = arr
        jsndta["ifint"] = True
        jsndta["type"] = "int"
        writetojson.addtovars(jsndta, i)
        
    for i in answerother:
        arr = []
        typevar = str(answerother[i][1])[8:]
        typevar2 = str(typevar[::-1][2:])[::-1]
        arr.append(i)
        arr.append(answerother[i][0])
        lastval = answerother[i][2][0][1]
        arr.append(lastval)
        arr2 = []
        for j in range(1, len(answerother[i][2])):
            arr2.append([answerother[i][2][j][0], lastval, answerother[i][2][j][1]])
            lastval = answerother[i][2][j][1]
        arr.append(arr2)
        jsndta = {}
        jsndta["timestamp"] = time.time()
        jsndta["report"] = arr
        jsndta["type"] = typevar2
        writetojson.addtovars(jsndta, i)

    arr = []
    arr.append(time.time()-stime)
    for i in nooftimesperline:
        arr.append([i, nooftimesperline[i]])
    jsndta = {}
    jsndta["timestamp"] = time.time()
    jsndta["report"] = arr
    writetojson.addtoothers(jsndta)
    writetojson.findata()
    writetojson.dumpjson()
    print("Written to json file!")




import glob, os
from PIL import Image, ImageDraw, ImageFont
import json
import copy
import textwrap
import cv2 as cv
import yaml
from moviepy.editor import *

dta = {}

def convertogif():
    clip = (VideoFileClip("DebuggerVideo.avi"))
    clip.write_gif("DebuggerVideo.gif")

def generatevid(pth):
    imgs = []
    with open(pth, 'r') as f:
        dta = json.load(f)
    pth2style = input("Enter the path to the yaml file: ")
    try:
        with open(pth2style) as f:
            style = yaml.load(f, Loader=yaml.FullLoader)
    except:
        style = {'introtext_time': 5, 'fps': 1, 'watermark': False, 'fontsz': 20, 'introtext': 'Code written by knightron0', 'font_path': 'fonts/hack.ttf', 'width': 1920, 'height': 1080, 'mute': ["a", "b"],'textwrap': 100, 'ptr': [["i", "arr"]]}
    height = style["height"]
    width = style["width"]
    ifmute = {}
    pnter = {}
    for j in style["mute"]:
        ifmute[j] = True
    for j in style["ptr"]:
        pnter[j[0]] = j[1]

    src = dta["source"].splitlines()
    fntsz = style["fontsz"]
    curr = -1
    strtln = dta["lines"]["1"]["report"][0]
    allvars = {}
    variablevalues = {}
    length = len(dta["lines"])
    img = Image.new('RGB', (width, height), color = (0, 0, 0))
    fnt = ImageFont.truetype(style["font_path"], fntsz)
    d = ImageDraw.Draw(img)
    w, h = d.textsize(style["introtext"], font=fnt)
    d.text(((width-w)/2, (height-h)/2), style["introtext"], fill=(255,255,255), font = fnt)
    num = 1

    for i in range(style["fps"]*style["introtext_time"]):
        imgnm = 'img' + str(num) + ".png"
        img.save(imgnm)
        img2 = cv.imread(imgnm)
        imgs.append(img2)
        num+=1
    tms = int(height//fntsz)
    till = 0
    outputtillnow = []
    oouttill = 0
    for j in range(1, len(dta["lines"])+1):
        img = Image.new('RGB', (width, height), color = (0, 0, 0))
        fnt = ImageFont.truetype(style["font_path"], fntsz)
        d = ImageDraw.Draw(img)
        if(dta["lines"][str(min(j+1, length))]["report"][10] != ""):
            outputtillnow.append(dta["lines"][str(j+1)]["report"][10])
        d.line((width/2,0, width/2,height), fill=(255,255,255), width=5)
        curr = dta["lines"][str(j)]["report"][0]-strtln+1
        for i in range(till, min(len(src), till + tms)):
            if(i == curr):
                d.rectangle(((0,((i-till)*fntsz)), (width/2,((i-till)*fntsz)+fntsz)), fill=(58, 61, 72))
                d.text((0,((i-till)*fntsz)),src[i], fill=(255,255,255), font = fnt)
            else:
                d.text((0,((i-till)*fntsz)),src[i], fill=(255,255,255), font = fnt)
        if curr >= (tms//2) and (len(src)-curr) >= (tms//2):
            till+=1
        fnt = ImageFont.truetype(style["font_path"], fntsz)
        if dta["lines"][str(min(j+1, length))]["report"][2] == 1:
            ln = "This line has been executed %s time and the time spent till now on this" % (1)
        else:
            ln = "This line has been executed %s times and the time spent till now on this" % (dta["lines"][str(min(j+1, length))]["report"][2])
        ln2 = "line is {:f} seconds".format(dta["lines"][str(min(j+1, length))]["report"][3])
        d.text((width/2+fntsz, fntsz),ln, fill=(255,255,255), font = fnt)
        d.text((width/2+fntsz, fntsz+fntsz),ln2, fill=(255,255,255), font = fnt)
        d.line((width/2, fntsz*4, width,fntsz*4), fill=(255,255,255), width=5)
        lftlnotpt = int((85*(height))/100)
        lfttms = int((height-lftlnotpt-fntsz)/fntsz)-1
        d.line((width/2, lftlnotpt, width,lftlnotpt), fill=(255,255,255), width=5)
        for i in range(max(len(outputtillnow) - lfttms, 0), len(outputtillnow)):
            d.text((width/2+fntsz, lftlnotpt + fntsz + ((i-max(len(outputtillnow) - lfttms, 1)) * fntsz)),outputtillnow[i], fill=(255,255,255), font = fnt)
        didmodify = {}
        for i in dta["lines"][str(min(j+1, length))]["report"][7]:
            allvars[i[0]] = [i[0], None, i[1]]
            variablevalues[i[0]] = i[1]
            didmodify[i[0]] = True

        for i in dta["lines"][str(min(j+1, length))]["report"][9]:
            allvars[i[0]] = [i[0], None, i[1]]
            didmodify[i[0]] = True

        for i in dta["lines"][str(min(j+1, length))]["report"][8]:
            allvars[i[0]] = [i[0], i[1], i[2]]
            didmodify[i[0]] = True

        for i in dta["lines"][str(min(j+1, length))]["report"][6]:
            temp = variablevalues[i[0]].copy()
            temp[i[1]] = i[3]
            allvars[i[0]] = [i[0], variablevalues[i[0]], temp]
            variablevalues[i[0]] = temp.copy()
            didmodify[i[0]] = True

        for i in dta["lines"][str(min(j+1, length))]["report"][5]:
            temp = variablevalues[i[0]].copy()
            temp[i[0]].append(i[1])
            allvars[i[0]] = [i[0], variablevalues[i[0]], temp]
            variablevalues[i[0]] = temp.copy()
            didmodify[i[0]] = True
        stp = 0
        yval = {}
        xval = {}
        for i in allvars:
            if i in didmodify and i not in ifmute:
                if(didmodify[i] == True):
                    if(allvars[i][1] != None):
                        if i not in pnter:
                            st = "The variable " + allvars[i][0] + " was changed, previous value " +  str(allvars[i][1]) + " and current value " + str(allvars[i][2])
                        else:
                            st = "The variable " + allvars[i][0]  + ", points to " +  pnter[i] + ", was changed, previous value " +  str(allvars[i][1]) + " and current value " + str(allvars[i][2])
                        lines = textwrap.wrap(st, width=70)
                        for k in lines:
                            d.text((width/2+fntsz, ((fntsz*4)+10) + (stp*fntsz)),k, fill=(0,200,0), font = fnt)
                            w, h = d.textsize(k, font=fnt)
                            stp+=1
                        yval[i] = ((fntsz*4)+10) + (stp*fntsz)- (fntsz/2)
                        xval[i] = width/2+fntsz + w + 20
                    else:
                        if i not in pnter:
                            st = "The variable " + allvars[i][0] + " was created, current value " + str(allvars[i][2])
                        else:
                            st = "The variable " + allvars[i][0] + ", points to " +  pnter[i] + ", was created, current value " + str(allvars[i][2])
                        lines = textwrap.wrap(st, width=70)
                        for k in lines:
                            d.text((width/2+fntsz, ((fntsz*4)+10) + (stp*fntsz)),k, fill=(69, 243, 226), font = fnt)
                            stp+=1
                            w, h = d.textsize(k, font=fnt)
                        yval[i] = ((fntsz*4)+10) + (stp*fntsz)- (fntsz/2)
                        xval[i] = width/2+fntsz + w + 20
            elif i not in didmodify and i not in ifmute:
                if i not in pnter:
                    st = "The variable " + allvars[i][0] + ", current value " + str(allvars[i][2])
                else:
                    st = "The variable " + allvars[i][0] + ", points to " +  pnter[i] + ", current value " + str(allvars[i][2])
                lines = textwrap.wrap(st, width=70)
                for k in lines:
                    d.text((width/2+fntsz, ((fntsz*4)+10) + (stp*fntsz)),k, fill=(255,255,255), font = fnt)
                    stp+=1
                    w, h = d.textsize(k, font=fnt)
                yval[i] = ((fntsz*4)+10) + (stp*fntsz) - (fntsz/2)
                xval[i] = width/2+fntsz + w + 20
        for i in pnter:
            if i in allvars and pnter[i] in allvars:
                d.line((width/2+ fntsz/2, yval[i],width/2+ fntsz,yval[i]), fill=(255,0,0), width=2)
                d.line((width/2+ fntsz/2, yval[i],width/2+ fntsz/2,yval[pnter[i]]), fill=(255,0,0), width=2)
                d.line((width/2+ fntsz/2,yval[pnter[i]],width/2+ fntsz,yval[pnter[i]]), fill=(255,0,0), width=2)
        if style["watermark"] == True:
            txt = "Generated by " + os.path.basename(__file__)
            w, h = d.textsize(txt, font=fnt)
            d.text((width-w, height-h), txt, fill=(255,255,255), font = fnt)
        imgnm = 'img' + str(num) + ".png"
        img.save(imgnm)
        img2 = cv.imread(imgnm)
        height, width, layers = img2.shape
        size = (width,height)
        imgs.append(img2)
        num += 1
    out = cv.VideoWriter('DebuggerVideo.avi',cv.VideoWriter_fourcc(*'DIVX'), style["fps"], size)
    imglast = imgs[len(imgs)-1]
    for i in range(5):
        imgs.append(imglast)
    for i in range(len(imgs)):
        out.write(imgs[i])
    out.release()
    for file in glob.glob("*.png"):
        os.remove(file)
    
    print("Video Saved in this folder with name DebuggerVideo.avi")
# generatevid("data.json")


import json

data = {}
data2 = {}

lines = {}
variables = {}
def addtolines(jsnd, step):
    lines[step] = jsnd

def addtovars(jsnd, step):
    variables[step] = jsnd

def addtoothers(jsnd):
    data["others"] = jsnd

def findata():
    data["variables"] = variables
    data["lines"] = lines

def dumpjson():
    with open('data.json', 'w') as fp:
        json.dump(data, fp)

def addtosource(srccode):
    data["source"] = srccode


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cimico", 
    version="1.0.2",
    author="knightron0",
    entry_points = {
        'console_scripts': ['cimico = cimico.main:main'],
    },
    author_email="sarthak.robo@gmail.com",
    description="A simple python debugger that generates a video of the program running along with valuable information.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/CCExtractor/cimico",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=['pillow', 'opencv-python', 'moviepy'],
)